// packages/babel/src/transforms/callExpression.ts
import { types as t2 } from "@babel/core";

// packages/babel/src/utils.ts
import { types as t } from "@babel/core";
import { addNamed } from "@babel/helper-module-imports";
function registerImportMethod(path, name, moduleName = "@macaron-css/core") {
  const imports = path.scope.getProgramParent().macaronData.imports || (path.scope.getProgramParent().macaronData.imports = /* @__PURE__ */ new Map());
  if (!imports.has(`${moduleName}:${name}`)) {
    let id = addNamed(path, name, moduleName);
    imports.set(`${moduleName}:${name}`, id);
    return id;
  } else {
    let iden = imports.get(`${moduleName}:${name}`);
    return t.cloneNode(iden);
  }
}
function getNearestIdentifier(path) {
  let currentPath = path;
  while (currentPath.parentPath !== null) {
    if (currentPath.isIdentifier()) {
      return currentPath;
    }
    let id = currentPath.get("id");
    if (!Array.isArray(id)) {
      if (id.isIdentifier())
        return id;
      if (id.isArrayPattern()) {
        for (const el of id.get("elements")) {
          if (el.isIdentifier())
            return el;
        }
      }
    }
    let key = currentPath.get("key");
    if (!Array.isArray(key) && key.isIdentifier()) {
      return key;
    }
    currentPath = currentPath.parentPath;
  }
  return null;
}
var extractionAPIs = [
  // @macaron-css/core
  "macaron$",
  // @vanilla-extract/css
  "style",
  "styleVariants",
  "globalStyle",
  "createTheme",
  "createGlobalTheme",
  "createThemeContract",
  "createGlobalThemeContract",
  "assignVars",
  "createVar",
  "fallbackVar",
  "fontFace",
  "globalFontFace",
  "keyframes",
  "globalKeyframes",
  "style",
  "styleVariants",
  "globalStyle",
  "createTheme",
  "createGlobalTheme",
  "createThemeContract",
  "createGlobalThemeContract",
  "assignVars",
  "createVar",
  "fallbackVar",
  "fontFace",
  "globalFontFace",
  "keyframes",
  "globalKeyframes",
  // @vanilla-extract/recipes
  "recipe"
];

// packages/babel/src/transforms/callExpression.ts
function transformCallExpression(callPath, _callState) {
  const callee = callPath.get("callee");
  if (!extractionAPIs.some(
    (api) => callee.referencesImport("@macaron-css/core", api)
  )) {
    return;
  }
  const programParent = callPath.scope.getProgramParent();
  if (callPath.node.leadingComments?.some(
    (comment) => comment.value.trim() === "macaron-ignore"
  ) || callPath.parent?.leadingComments?.some(
    (comment) => comment.value.trim() === "macaron-ignore"
  )) {
    const bindings2 = getBindings(callPath.get("callee"));
    for (const binding of bindings2) {
      programParent.macaronData.nodes.push(findRootBinding(binding).node);
    }
    return;
  }
  const nearestIdentifier = getNearestIdentifier(callPath);
  const ident = nearestIdentifier ? programParent.generateUidIdentifier(
    `$macaron$$${nearestIdentifier.node.name}`
  ) : programParent.generateUidIdentifier("$macaron$$unknown");
  const importedIdent = registerImportMethod(
    callPath,
    ident.name,
    programParent.macaronData.cssFile
  );
  const bindings = getBindings(callPath);
  for (const binding of bindings) {
    programParent.macaronData.nodes.push(findRootBinding(binding).node);
  }
  programParent.macaronData.nodes.push(
    t2.exportNamedDeclaration(
      t2.variableDeclaration("var", [t2.variableDeclarator(ident, callPath.node)])
    )
  );
  programParent.macaronData.nodes.push(
    t2.variableDeclaration("var", [t2.variableDeclarator(importedIdent, ident)])
  );
  callPath.replaceWith(importedIdent);
}
function findRootBinding(path) {
  let rootPath;
  if (!("parent" in path) || path.parentPath?.isProgram()) {
    rootPath = path;
  } else {
    rootPath = path.parentPath;
  }
  return rootPath;
}
function getBindings(path) {
  const programParent = path.scope.getProgramParent();
  const bindings = [];
  path.traverse({
    Expression(expressionPath, state) {
      if (!expressionPath.isIdentifier())
        return;
      const binding = path.scope.getBinding(expressionPath);
      if (!binding || programParent.macaronData.bindings.includes(binding.path) || bindings.includes(binding.path))
        return;
      const rootBinding = findRootBinding(binding.path);
      if (path === rootBinding) {
        bindings.push(binding.path);
        return;
      }
      const bindingOfBindings = getBindings(rootBinding);
      bindings.push(...bindingOfBindings, binding.path);
    }
  });
  programParent.macaronData.bindings.push(...bindings);
  return bindings;
}

// packages/babel/src/transforms/postprocess.ts
import { types as t3 } from "@babel/core";
import * as generator from "@babel/generator";
function postprocess(path, state) {
  const programParent = path.scope;
  const cssExtract = generator.default(
    t3.program(programParent.macaronData.nodes)
  ).code;
  state.opts.result = [programParent.macaronData.cssFile, cssExtract];
}

// packages/babel/src/transforms/preprocess.ts
import hash from "@emotion/hash";
function preprocess(path, state) {
  path.scope.macaronData = {
    imports: /* @__PURE__ */ new Map(),
    cssFile: `extracted_${hash(path.toString())}.css.ts`,
    nodes: [],
    bindings: []
  };
}

// packages/babel/src/styled.ts
import { types as t4 } from "@babel/core";
function styledComponentsPlugin() {
  return {
    name: "macaron-css-babel:styled",
    visitor: {
      Program: {
        enter(path) {
          path.scope.macaronData ??= {};
          path.traverse({
            CallExpression(callPath) {
              const callee = callPath.get("callee");
              const isSolidAdapter = callee.referencesImport(
                "@macaron-css/solid",
                "styled"
              );
              const isReactAdapter = callee.referencesImport(
                "@macaron-css/react",
                "styled"
              );
              if (!isSolidAdapter && !isReactAdapter)
                return;
              const runtimeImport = isSolidAdapter ? "@macaron-css/solid/runtime" : "@macaron-css/react/runtime";
              const [tag, styles, ...args] = callPath.node.arguments;
              const styledIdent = registerImportMethod(
                callPath,
                "$$styled",
                runtimeImport
              );
              const recipeIdent = registerImportMethod(
                callPath,
                "recipe",
                "@macaron-css/core"
              );
              const recipeCallExpression = t4.callExpression(recipeIdent, [
                t4.cloneNode(styles)
              ]);
              t4.addComments(
                recipeCallExpression,
                "leading",
                callPath.node.leadingComments ?? []
              );
              const callExpression = t4.callExpression(styledIdent, [
                t4.cloneNode(tag),
                recipeCallExpression,
                ...args
              ]);
              t4.addComment(callExpression, "leading", " @__PURE__ ");
              callPath.replaceWith(callExpression);
              path.scope.crawl();
            }
          });
        }
      }
    }
  };
}

// packages/babel/src/index.ts
function macaronBabelPlugin() {
  return {
    name: "macaron-css-babel",
    visitor: {
      Program: {
        enter: preprocess,
        exit: postprocess
      },
      CallExpression: transformCallExpression
    }
  };
}
export {
  macaronBabelPlugin,
  styledComponentsPlugin as macaronStyledComponentsPlugin
};
//# sourceMappingURL=index.mjs.map